@@ -10,10 +10,6 @@
 
 #include <libedataserver/libedataserver.h>
 
-#define CAMEL_EWS_SETTINGS_GET_PRIVATE(obj) \
-	(G_TYPE_INSTANCE_GET_PRIVATE \
-	((obj), CAMEL_TYPE_EWS_SETTINGS, CamelEwsSettingsPrivate))
-
 struct _CamelEwsSettingsPrivate {
 	GMutex property_lock;
 	gboolean check_all;
@@ -37,6 +33,8 @@ struct _CamelEwsSettingsPrivate {
 	gchar *oauth2_tenant;
 	gchar *oauth2_client_id;
 	gchar *oauth2_redirect_uri;
+	gchar *oauth2_resource_uri;
+	gchar *oauth2_endpoint_host;
 };
 
 enum {
@@ -65,16 +63,15 @@ enum {
 	PROP_OAUTH2_TENANT,
 	PROP_OAUTH2_CLIENT_ID,
 	PROP_OAUTH2_REDIRECT_URI,
+	PROP_OAUTH2_RESOURCE_URI,
+	PROP_OAUTH2_ENDPOINT_HOST,
 	PROP_SHOW_PUBLIC_FOLDERS,
 	PROP_CONCURRENT_CONNECTIONS
 };
 
-G_DEFINE_TYPE_WITH_CODE (
-	CamelEwsSettings,
-	camel_ews_settings,
-	CAMEL_TYPE_OFFLINE_SETTINGS,
-	G_IMPLEMENT_INTERFACE (
-		CAMEL_TYPE_NETWORK_SETTINGS, NULL))
+G_DEFINE_TYPE_WITH_CODE (CamelEwsSettings, camel_ews_settings, CAMEL_TYPE_OFFLINE_SETTINGS,
+	G_ADD_PRIVATE (CamelEwsSettings)
+	G_IMPLEMENT_INTERFACE (CAMEL_TYPE_NETWORK_SETTINGS, NULL))
 
 static gboolean
 ews_settings_transform_host_url_to_host_cb (GBinding *binding,
@@ -256,6 +253,18 @@ ews_settings_set_property (GObject *object,
 				g_value_get_string (value));
 			return;
 
+		case PROP_OAUTH2_RESOURCE_URI:
+			camel_ews_settings_set_oauth2_resource_uri (
+				CAMEL_EWS_SETTINGS (object),
+				g_value_get_string (value));
+			return;
+
+		case PROP_OAUTH2_ENDPOINT_HOST:
+			camel_ews_settings_set_oauth2_endpoint_host (
+				CAMEL_EWS_SETTINGS (object),
+				g_value_get_string (value));
+			return;
+
 		case PROP_SHOW_PUBLIC_FOLDERS:
 			camel_ews_settings_set_show_public_folders (
 				CAMEL_EWS_SETTINGS (object),
@@ -447,6 +456,20 @@ ews_settings_get_property (GObject *object,
 				CAMEL_EWS_SETTINGS (object)));
 			return;
 
+		case PROP_OAUTH2_RESOURCE_URI:
+			g_value_take_string (
+				value,
+				camel_ews_settings_dup_oauth2_resource_uri (
+				CAMEL_EWS_SETTINGS (object)));
+			return;
+
+		case PROP_OAUTH2_ENDPOINT_HOST:
+			g_value_take_string (
+				value,
+				camel_ews_settings_dup_oauth2_endpoint_host (
+				CAMEL_EWS_SETTINGS (object)));
+			return;
+
 		case PROP_SHOW_PUBLIC_FOLDERS:
 			g_value_set_boolean (
 				value,
@@ -468,22 +491,22 @@ ews_settings_get_property (GObject *object,
 static void
 ews_settings_finalize (GObject *object)
 {
-	CamelEwsSettingsPrivate *priv;
+	CamelEwsSettings *ews_settings = CAMEL_EWS_SETTINGS (object);
 
-	priv = CAMEL_EWS_SETTINGS_GET_PRIVATE (object);
+	g_mutex_clear (&ews_settings->priv->property_lock);
 
-	g_mutex_clear (&priv->property_lock);
-
-	g_free (priv->email);
-	g_free (priv->gal_uid);
-	g_free (priv->hosturl);
-	g_free (priv->oaburl);
-	g_free (priv->oal_selected);
-	g_free (priv->impersonate_user);
-	g_free (priv->user_agent);
-	g_free (priv->oauth2_tenant);
-	g_free (priv->oauth2_client_id);
-	g_free (priv->oauth2_redirect_uri);
+	g_free (ews_settings->priv->email);
+	g_free (ews_settings->priv->gal_uid);
+	g_free (ews_settings->priv->hosturl);
+	g_free (ews_settings->priv->oaburl);
+	g_free (ews_settings->priv->oal_selected);
+	g_free (ews_settings->priv->impersonate_user);
+	g_free (ews_settings->priv->user_agent);
+	g_free (ews_settings->priv->oauth2_tenant);
+	g_free (ews_settings->priv->oauth2_client_id);
+	g_free (ews_settings->priv->oauth2_redirect_uri);
+	g_free (ews_settings->priv->oauth2_resource_uri);
+	g_free (ews_settings->priv->oauth2_endpoint_host);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (camel_ews_settings_parent_class)->finalize (object);
@@ -494,8 +517,6 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 {
 	GObjectClass *object_class;
 
-	g_type_class_add_private (class, sizeof (CamelEwsSettingsPrivate));
-
 	object_class = G_OBJECT_CLASS (class);
 	object_class->set_property = ews_settings_set_property;
 	object_class->get_property = ews_settings_get_property;
@@ -730,7 +751,7 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 			"oauth2-tenant",
 			"OAuth2 Tenant",
 			"OAuth2 Tenant to use, only if override-oauth2 is TRUE, otherwise the compile-time value is used",
-			NULL,
+			OFFICE365_TENANT,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
@@ -742,7 +763,7 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 			"oauth2-client-id",
 			"OAuth2 Client ID",
 			"OAuth2 Client-ID to use, only if override-oauth2 is TRUE, otherwise the compile-time value is used",
-			NULL,
+			OFFICE365_CLIENT_ID,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
@@ -754,11 +775,35 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 			"oauth2-redirect-uri",
 			"OAuth2 Redirect URI",
 			"OAuth2 Redirect URI to use, only if override-oauth2 is TRUE, otherwise the compile-time value is used",
+			OFFICE365_REDIRECT_URI,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_OAUTH2_RESOURCE_URI,
+		g_param_spec_string (
+			"oauth2-resource-uri",
+			"OAuth2 Resource URI",
+			"OAuth2 Resource URI to use, only if override-oauth2 is TRUE, otherwise the compile-time value is used",
 			NULL,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_OAUTH2_ENDPOINT_HOST,
+		g_param_spec_string (
+			"oauth2-endpoint-host",
+			"OAuth2 Endpoint Host",
+			"OAuth2 endpoint host to use, only if override-oauth2 is TRUE, otherwise the compile-time value is used",
+			OFFICE365_ENDPOINT_HOST,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property (
 		object_class,
 		PROP_SHOW_PUBLIC_FOLDERS,
@@ -790,7 +835,7 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 static void
 camel_ews_settings_init (CamelEwsSettings *settings)
 {
-	settings->priv = CAMEL_EWS_SETTINGS_GET_PRIVATE (settings);
+	settings->priv = camel_ews_settings_get_instance_private (settings);
 	g_mutex_init (&settings->priv->property_lock);
 
 	e_binding_bind_property_full (settings, "hosturl",
@@ -802,6 +847,22 @@ camel_ews_settings_init (CamelEwsSettings *settings)
 				     NULL);
 }
 
+void
+camel_ews_settings_lock (CamelEwsSettings *settings)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	g_mutex_lock (&settings->priv->property_lock);
+}
+
+void
+camel_ews_settings_unlock (CamelEwsSettings *settings)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	g_mutex_unlock (&settings->priv->property_lock);
+}
+
 /**
  * camel_ews_settings_get_auth_mechanism:
  * @settings: a #CamelEwsSettings
@@ -939,12 +1000,12 @@ camel_ews_settings_dup_email (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_email (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -955,17 +1016,17 @@ camel_ews_settings_set_email (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->email, email) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->email);
 	settings->priv->email = e_util_strdup_strip (email);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "email");
 }
@@ -1072,12 +1133,12 @@ camel_ews_settings_dup_gal_uid (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_gal_uid (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1088,17 +1149,17 @@ camel_ews_settings_set_gal_uid (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->gal_uid, gal_uid) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->gal_uid);
 	settings->priv->gal_uid = e_util_strdup_strip (gal_uid);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "gal-uid");
 }
@@ -1119,12 +1180,12 @@ camel_ews_settings_dup_hosturl (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_hosturl (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1135,17 +1196,17 @@ camel_ews_settings_set_hosturl (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->hosturl, hosturl) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->hosturl);
 	settings->priv->hosturl = e_util_strdup_strip (hosturl);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "hosturl");
 }
@@ -1166,12 +1227,12 @@ camel_ews_settings_dup_oaburl (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_oaburl (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1182,17 +1243,17 @@ camel_ews_settings_set_oaburl (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->oaburl, oaburl) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->oaburl);
 	settings->priv->oaburl = e_util_strdup_strip (oaburl);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "oaburl");
 }
@@ -1235,12 +1296,12 @@ camel_ews_settings_dup_oal_selected (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_oal_selected (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1251,17 +1312,17 @@ camel_ews_settings_set_oal_selected (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->oal_selected, oal_selected) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->oal_selected);
 	settings->priv->oal_selected = e_util_strdup_strip (oal_selected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "oal-selected");
 }
@@ -1326,12 +1387,12 @@ camel_ews_settings_dup_impersonate_user (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_impersonate_user (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1342,17 +1403,17 @@ camel_ews_settings_set_impersonate_user (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->impersonate_user, impersonate_user) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->impersonate_user);
 	settings->priv->impersonate_user = e_util_strdup_strip (impersonate_user);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "impersonate-user");
 }
@@ -1395,12 +1456,12 @@ camel_ews_settings_dup_user_agent (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_user_agent (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1411,17 +1472,17 @@ camel_ews_settings_set_user_agent (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->user_agent, user_agent) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->user_agent);
 	settings->priv->user_agent = e_util_strdup_strip (user_agent);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "user-agent");
 }
@@ -1464,12 +1525,12 @@ camel_ews_settings_dup_oauth2_tenant (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_oauth2_tenant (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1480,17 +1541,17 @@ camel_ews_settings_set_oauth2_tenant (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->oauth2_tenant, tenant) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->oauth2_tenant);
 	settings->priv->oauth2_tenant = e_util_strdup_strip (tenant);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "oauth2-tenant");
 }
@@ -1511,12 +1572,12 @@ camel_ews_settings_dup_oauth2_client_id (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_oauth2_client_id (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1527,17 +1588,17 @@ camel_ews_settings_set_oauth2_client_id (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->oauth2_client_id, client_id) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->oauth2_client_id);
 	settings->priv->oauth2_client_id = e_util_strdup_strip (client_id);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "oauth2-client-id");
 }
@@ -1558,12 +1619,12 @@ camel_ews_settings_dup_oauth2_redirect_uri (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	protected = camel_ews_settings_get_oauth2_redirect_uri (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	return duplicate;
 }
@@ -1574,21 +1635,115 @@ camel_ews_settings_set_oauth2_redirect_uri (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (&settings->priv->property_lock);
+	camel_ews_settings_lock (settings);
 
 	if (g_strcmp0 (settings->priv->oauth2_redirect_uri, redirect_uri) == 0) {
-		g_mutex_unlock (&settings->priv->property_lock);
+		camel_ews_settings_unlock (settings);
 		return;
 	}
 
 	g_free (settings->priv->oauth2_redirect_uri);
 	settings->priv->oauth2_redirect_uri = e_util_strdup_strip (redirect_uri);
 
-	g_mutex_unlock (&settings->priv->property_lock);
+	camel_ews_settings_unlock (settings);
 
 	g_object_notify (G_OBJECT (settings), "oauth2-redirect-uri");
 }
 
+const gchar *
+camel_ews_settings_get_oauth2_resource_uri (CamelEwsSettings *settings)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
+
+	return settings->priv->oauth2_resource_uri;
+}
+
+gchar *
+camel_ews_settings_dup_oauth2_resource_uri (CamelEwsSettings *settings)
+{
+	const gchar *protected;
+	gchar *duplicate;
+
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
+
+	camel_ews_settings_lock (settings);
+
+	protected = camel_ews_settings_get_oauth2_resource_uri (settings);
+	duplicate = g_strdup (protected);
+
+	camel_ews_settings_unlock (settings);
+
+	return duplicate;
+}
+
+void
+camel_ews_settings_set_oauth2_resource_uri (CamelEwsSettings *settings,
+					    const gchar *resource_uri)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	camel_ews_settings_lock (settings);
+
+	if (g_strcmp0 (settings->priv->oauth2_resource_uri, resource_uri) == 0) {
+		camel_ews_settings_unlock (settings);
+		return;
+	}
+
+	g_free (settings->priv->oauth2_resource_uri);
+	settings->priv->oauth2_resource_uri = e_util_strdup_strip (resource_uri);
+
+	camel_ews_settings_unlock (settings);
+
+	g_object_notify (G_OBJECT (settings), "oauth2-resource-uri");
+}
+
+const gchar *
+camel_ews_settings_get_oauth2_endpoint_host (CamelEwsSettings *settings)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
+
+	return settings->priv->oauth2_endpoint_host;
+}
+
+gchar *
+camel_ews_settings_dup_oauth2_endpoint_host (CamelEwsSettings *settings)
+{
+	const gchar *protected;
+	gchar *duplicate;
+
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
+
+	camel_ews_settings_lock (settings);
+
+	protected = camel_ews_settings_get_oauth2_endpoint_host (settings);
+	duplicate = g_strdup (protected);
+
+	camel_ews_settings_unlock (settings);
+
+	return duplicate;
+}
+
+void
+camel_ews_settings_set_oauth2_endpoint_host (CamelEwsSettings *settings,
+					     const gchar *endpoint_host)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	camel_ews_settings_lock (settings);
+
+	if (g_strcmp0 (settings->priv->oauth2_endpoint_host, endpoint_host) == 0) {
+		camel_ews_settings_unlock (settings);
+		return;
+	}
+
+	g_free (settings->priv->oauth2_endpoint_host);
+	settings->priv->oauth2_endpoint_host = e_util_strdup_strip (endpoint_host);
+
+	camel_ews_settings_unlock (settings);
+
+	g_object_notify (G_OBJECT (settings), "oauth2-endpoint-host");
+}
+
 gboolean
 camel_ews_settings_get_show_public_folders (CamelEwsSettings *settings)
 {
