@@ -13,14 +13,7 @@
 #include "e-ews-notification.h"
 #include "e-soup-auth-negotiate.h"
 
-#define E_EWS_NOTIFICATION_GET_PRIVATE(obj)\
-	(G_TYPE_INSTANCE_GET_PRIVATE \
-	((obj), E_TYPE_EWS_NOTIFICATION, EEwsNotificationPrivate))
-
-G_DEFINE_TYPE (EEwsNotification, e_ews_notification, G_TYPE_OBJECT)
-
 struct _EEwsNotificationPrivate {
-	GMutex thread_lock;
 	SoupSession *soup_session;
 	GWeakRef connection_wk;
 	GByteArray *chunk;
@@ -28,6 +21,8 @@ struct _EEwsNotificationPrivate {
 	gchar *last_subscription_id; /* guarded by the caller, because it can be set only after construct */
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (EEwsNotification, e_ews_notification, G_TYPE_OBJECT)
+
 enum {
 	PROP_0,
 	PROP_CONNECTION
@@ -169,6 +164,12 @@ ews_notification_constructed (GObject *object)
 			camel_ews_settings_get_auth_mechanism (ews_settings));
 
 		g_object_unref (ews_settings);
+
+		e_binding_bind_property (
+			cnc, "proxy-resolver",
+			notif->priv->soup_session, "proxy-resolver",
+			G_BINDING_SYNC_CREATE);
+
 		g_object_unref (cnc);
 	}
 }
@@ -176,24 +177,18 @@ ews_notification_constructed (GObject *object)
 static void
 ews_notification_dispose (GObject *object)
 {
-	EEwsNotificationPrivate *priv;
+	EEwsNotification *notif = E_EWS_NOTIFICATION (object);
 
-	priv = E_EWS_NOTIFICATION_GET_PRIVATE (object);
+	if (notif->priv->cancellable != NULL)
+		g_cancellable_cancel (notif->priv->cancellable);
 
-	if (priv->cancellable != NULL)
-		g_cancellable_cancel (priv->cancellable);
-
-	if (priv->soup_session != NULL) {
-		g_signal_handlers_disconnect_by_func (
-			priv->soup_session,
-			ews_notification_authenticate, object);
-		g_clear_object (&priv->soup_session);
+	if (notif->priv->soup_session != NULL) {
+		g_signal_handlers_disconnect_by_func (notif->priv->soup_session, ews_notification_authenticate, object);
+		g_clear_object (&notif->priv->soup_session);
 	}
 
-	if (priv->cancellable != NULL)
-		g_clear_object (&priv->cancellable);
-
-	g_weak_ref_set (&priv->connection_wk, NULL);
+	g_clear_object (&notif->priv->cancellable);
+	g_weak_ref_set (&notif->priv->connection_wk, NULL);
 
 	/* Chain up to parent's dispose() method. */
 	G_OBJECT_CLASS (e_ews_notification_parent_class)->dispose (object);
@@ -208,7 +203,6 @@ ews_notification_finalize (GObject *object)
 
 	g_weak_ref_clear (&notif->priv->connection_wk);
 	g_free (notif->priv->last_subscription_id);
-	g_mutex_clear (&notif->priv->thread_lock);
 
 	/* Chain up to parent's method. */
 	G_OBJECT_CLASS (e_ews_notification_parent_class)->finalize (object);
@@ -219,8 +213,6 @@ e_ews_notification_class_init (EEwsNotificationClass *class)
 {
 	GObjectClass *object_class;
 
-	g_type_class_add_private (class, sizeof (EEwsNotificationPrivate));
-
 	object_class = G_OBJECT_CLASS (class);
 	object_class->set_property = ews_notification_set_property;
 	object_class->get_property = ews_notification_get_property;
@@ -255,12 +247,11 @@ e_ews_notification_init (EEwsNotification *notification)
 {
 	gint log_level;
 
-	notification->priv = E_EWS_NOTIFICATION_GET_PRIVATE (notification);
+	notification->priv = e_ews_notification_get_instance_private (notification);
 
 	g_weak_ref_init (&notification->priv->connection_wk, NULL);
-	g_mutex_init (&notification->priv->thread_lock);
 
-	notification->priv->soup_session = soup_session_new ();
+	notification->priv->soup_session = soup_session_sync_new ();
 
 	soup_session_add_feature_by_type (notification->priv->soup_session,
 					  SOUP_TYPE_COOKIE_JAR);
@@ -297,9 +288,6 @@ e_ews_notification_subscribe_folder_sync (EEwsNotification *notification,
 	g_return_val_if_fail (notification != NULL, FALSE);
 	g_return_val_if_fail (notification->priv != NULL, FALSE);
 
-	if (g_cancellable_is_cancelled (cancellable))
-		return FALSE;
-
 	cnc = e_ews_notification_ref_connection (notification);
 
 	/* Can happen during process shutdown */
@@ -845,9 +833,6 @@ e_ews_notification_get_events_sync (EEwsNotification *notification,
 	g_return_val_if_fail (notification != NULL, FALSE);
 	g_return_val_if_fail (notification->priv != NULL, FALSE);
 
-	if (g_cancellable_is_cancelled (cancellable))
-		return FALSE;
-
 	cnc = e_ews_notification_ref_connection (notification);
 
 	if (!cnc)
@@ -896,25 +881,20 @@ e_ews_notification_get_events_sync (EEwsNotification *notification,
 		return FALSE;
 	}
 
-	/* Unref it early, thus it doesn't keep the connection alive after all backends are freed */
-	g_object_unref (cnc);
-
 	cancel_handler_id = g_cancellable_connect (cancellable, G_CALLBACK (ews_notification_cancelled_cb),
 		notifcation_cancel_data_new (notification->priv->soup_session, SOUP_MESSAGE (msg)), notifcation_cancel_data_free);
 
-	if (g_cancellable_is_cancelled (cancellable))
-		status_code = SOUP_STATUS_CANCELLED;
-	else
-		status_code = soup_session_send_message (notification->priv->soup_session, SOUP_MESSAGE (msg));
+	status_code = soup_session_send_message (notification->priv->soup_session, SOUP_MESSAGE (msg));
 
 	if (cancel_handler_id > 0)
 		g_cancellable_disconnect (cancellable, cancel_handler_id);
 
 	ret = SOUP_STATUS_IS_SUCCESSFUL (status_code);
-	*out_fatal_error = SOUP_STATUS_IS_CLIENT_ERROR (status_code) || SOUP_STATUS_IS_SERVER_ERROR (status_code) || status_code == SOUP_STATUS_CANCELLED;
+	*out_fatal_error = SOUP_STATUS_IS_CLIENT_ERROR (status_code) || SOUP_STATUS_IS_SERVER_ERROR (status_code);
 
 	g_signal_handler_disconnect (msg, handler_id);
 	g_object_unref (msg);
+	g_object_unref (cnc);
 
 	return ret;
 }
@@ -923,7 +903,6 @@ static gpointer
 e_ews_notification_get_events_thread (gpointer user_data)
 {
 	EEwsNotificationThreadData *td = user_data;
-	EEwsConnection *cnc;
 	gchar *subscription_id = NULL;
 	gboolean ret, fatal_error = FALSE;
 
@@ -931,22 +910,6 @@ e_ews_notification_get_events_thread (gpointer user_data)
 	g_return_val_if_fail (td->notification != NULL, NULL);
 	g_return_val_if_fail (td->folders != NULL, NULL);
 
-	g_mutex_lock (&td->notification->priv->thread_lock);
-
-	cnc = e_ews_notification_ref_connection (td->notification);
-
-	if (cnc) {
-		GProxyResolver *proxy_resolver = NULL;
-
-		/* Skip GBinding here, due to the notification object can be freed in this thread,
-		   which can cause a crash in the GBinding code. */
-		g_object_get (cnc, "proxy-resolver", &proxy_resolver, NULL);
-		g_object_set (td->notification->priv->soup_session, "proxy-resolver", proxy_resolver, NULL);
-
-		g_clear_object (&proxy_resolver);
-		g_object_unref (cnc);
-	}
-
 	if (td->notification->priv->last_subscription_id) {
 		e_ews_notification_unsubscribe_folder_sync (td->notification, td->notification->priv->last_subscription_id);
 		g_clear_pointer (&td->notification->priv->last_subscription_id, g_free);
@@ -985,7 +948,6 @@ exit:
 		g_free (subscription_id);
 	}
 
-	g_mutex_unlock (&td->notification->priv->thread_lock);
 	g_slist_free_full (td->folders, g_free);
 	g_object_unref (td->cancellable);
 	g_object_unref (td->notification);
