@@ -27,6 +27,7 @@ struct _EMailConfigM365BackendPrivate {
 	GtkWidget *oauth2_tenant_entry;
 	GtkWidget *oauth2_client_id_entry;
 	GtkWidget *oauth2_redirect_uri_entry;
+	GtkWidget *oauth2_endpoint_host_entry;
 };
 
 G_DEFINE_DYNAMIC_TYPE_EXTENDED (EMailConfigM365Backend, e_mail_config_m365_backend, E_TYPE_MAIL_CONFIG_SERVICE_BACKEND, 0,
@@ -82,9 +83,14 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	ESourceExtension *extension;
 	ESourceAuthentication *auth_extension;
 	CamelSettings *settings;
+	CamelM365Settings *m365_settings;
 	GtkLabel *label;
 	GtkWidget *widget;
 	GtkWidget *container;
+	GtkWidget *expander;
+	GtkWidget *advanced_help;
+	GtkWidget *endpoint_host_label;
+	GtkWidget *redirect_uri_label;
 	const gchar *extension_name;
 	const gchar *text;
 	gchar *markup;
@@ -140,15 +146,25 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	gtk_grid_attach (GTK_GRID (container), widget, 1, 1, 1, 1);
 	gtk_widget_show (widget);
 
-	if (camel_m365_settings_get_use_impersonation (CAMEL_M365_SETTINGS (settings))) {
-		const gchar *impersonate_user = camel_m365_settings_get_impersonate_user (CAMEL_M365_SETTINGS (settings));
+	m365_settings = CAMEL_M365_SETTINGS (settings);
+
+	camel_m365_settings_lock (m365_settings);
+
+	if (camel_m365_settings_get_use_impersonation (m365_settings)) {
+		const gchar *impersonate_user = camel_m365_settings_get_impersonate_user (m365_settings);
 
 		if (impersonate_user && !*impersonate_user) {
-			camel_m365_settings_set_impersonate_user (CAMEL_M365_SETTINGS (settings), NULL);
-			camel_m365_settings_set_use_impersonation (CAMEL_M365_SETTINGS (settings), FALSE);
+			camel_m365_settings_unlock (m365_settings);
+
+			camel_m365_settings_set_impersonate_user (m365_settings, NULL);
+			camel_m365_settings_set_use_impersonation (m365_settings, FALSE);
+
+			camel_m365_settings_lock (m365_settings);
 		}
 	}
 
+	camel_m365_settings_unlock (m365_settings);
+
 	e_binding_bind_property (
 		settings, "use-impersonation",
 		widget, "active",
@@ -202,9 +218,7 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
 	g_free (markup);
 
-	/* Translators: 'Tenant' here means a term used by Microsoft to identify a company or organization in a Microsoft 365 world.
-	   You probably do not want to translate it. More for example here: https://powerbi.microsoft.com/en-us/blog/what-is-a-tenant/ */
-	widget = gtk_label_new_with_mnemonic (_("_Tenant:"));
+	widget = gtk_label_new_with_mnemonic (_("Application I_D:"));
 	gtk_widget_set_margin_left (widget, 12);
 	gtk_misc_set_alignment (GTK_MISC (widget), 1.0, 0.5);
 	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 0, 1, 1, 1);
@@ -219,6 +233,34 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	gtk_widget_set_hexpand (widget, TRUE);
 	gtk_label_set_mnemonic_widget (label, widget);
 	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 1, 1, 1, 1);
+	m365_backend->priv->oauth2_client_id_entry = widget;
+
+	e_binding_bind_property (
+		m365_backend->priv->oauth2_override_check, "active",
+		widget, "sensitive",
+		G_BINDING_SYNC_CREATE);
+
+	mail_config_m365_backend_set_oauth2_tooltip (widget, MICROSOFT365_CLIENT_ID,
+		_("There is not set any default application ID"),
+		g_strdup_printf (_("Default application ID is “%s”"), MICROSOFT365_CLIENT_ID));
+
+	/* Translators: 'Tenant ID' here means a term used by Microsoft to identify a company or organization in a Microsoft 365 world.
+	   You probably do not want to translate it. More for example here: https://powerbi.microsoft.com/en-us/blog/what-is-a-tenant/ */
+	widget = gtk_label_new_with_mnemonic (_("_Tenant ID:"));
+	gtk_widget_set_margin_left (widget, 12);
+	gtk_misc_set_alignment (GTK_MISC (widget), 1.0, 0.5);
+	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 0, 2, 1, 1);
+	label = GTK_LABEL (widget);
+
+	e_binding_bind_property (
+		m365_backend->priv->oauth2_override_check, "active",
+		widget, "sensitive",
+		G_BINDING_SYNC_CREATE);
+
+	widget = gtk_entry_new ();
+	gtk_widget_set_hexpand (widget, TRUE);
+	gtk_label_set_mnemonic_widget (label, widget);
+	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 1, 2, 1, 1);
 	m365_backend->priv->oauth2_tenant_entry = widget;
 
 	e_binding_bind_property (
@@ -234,11 +276,33 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 		   You probably do not want to translate it. More for example here: https://powerbi.microsoft.com/en-us/blog/what-is-a-tenant/ */
 		g_strdup_printf (_("Default tenant is “%s”"), MICROSOFT365_TENANT));
 
-	widget = gtk_label_new_with_mnemonic (_("Application I_D:"));
+	container = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, container, 0, 3, 2, 1);
+
+	widget = gtk_expander_new_with_mnemonic (_("_Advanced Settings"));
+	gtk_widget_set_margin_left (widget, 12);
+	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
+	expander = widget;
+
+	e_binding_bind_property (
+		m365_backend->priv->oauth2_override_check, "active",
+		widget, "sensitive",
+		G_BINDING_SYNC_CREATE);
+
+	markup = g_markup_printf_escaped ("(<a href=\"https://wiki.gnome.org/Apps/Evolution/EWS/OAuth2#Alternative_endpoints\">%s</a>)", _("Help…"));
+	widget = gtk_label_new (markup);
+	gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
+	g_free (markup);
+	advanced_help = widget;
+
+	widget = gtk_label_new_with_mnemonic (_("_Endpoint host:"));
 	gtk_widget_set_margin_left (widget, 12);
 	gtk_misc_set_alignment (GTK_MISC (widget), 1.0, 0.5);
-	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 0, 2, 1, 1);
+	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 0, 4, 1, 1);
 	label = GTK_LABEL (widget);
+	endpoint_host_label = widget;
 
 	e_binding_bind_property (
 		m365_backend->priv->oauth2_override_check, "active",
@@ -248,23 +312,26 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	widget = gtk_entry_new ();
 	gtk_widget_set_hexpand (widget, TRUE);
 	gtk_label_set_mnemonic_widget (label, widget);
-	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 1, 2, 1, 1);
-	m365_backend->priv->oauth2_client_id_entry = widget;
+	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 1, 4, 1, 1);
+	m365_backend->priv->oauth2_endpoint_host_entry = widget;
 
 	e_binding_bind_property (
 		m365_backend->priv->oauth2_override_check, "active",
 		widget, "sensitive",
 		G_BINDING_SYNC_CREATE);
 
-	mail_config_m365_backend_set_oauth2_tooltip (widget, MICROSOFT365_CLIENT_ID,
-		_("There is not set any default application ID"),
-		g_strdup_printf (_("Default application ID is “%s”"), MICROSOFT365_CLIENT_ID));
+	markup = g_strdup_printf (_("Default endpoint host is “%s”"), "login.microsoftonline.com");
+	mail_config_m365_backend_set_oauth2_tooltip (widget, OFFICE365_ENDPOINT_HOST,
+		markup,
+		g_strdup_printf (_("Default endpoint host is “%s”"), OFFICE365_ENDPOINT_HOST));
+	g_free (markup);
 
 	widget = gtk_label_new_with_mnemonic (_("_Redirect URI:"));
 	gtk_widget_set_margin_left (widget, 12);
 	gtk_misc_set_alignment (GTK_MISC (widget), 1.0, 0.5);
-	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 0, 3, 1, 1);
+	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 0, 5, 1, 1);
 	label = GTK_LABEL (widget);
+	redirect_uri_label = widget;
 
 	e_binding_bind_property (
 		m365_backend->priv->oauth2_override_check, "active",
@@ -274,7 +341,7 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 	widget = gtk_entry_new ();
 	gtk_widget_set_hexpand (widget, TRUE);
 	gtk_label_set_mnemonic_widget (label, widget);
-	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 1, 3, 1, 1);
+	gtk_grid_attach (m365_backend->priv->oauth2_settings_grid, widget, 1, 5, 1, 1);
 	m365_backend->priv->oauth2_redirect_uri_entry = widget;
 
 	e_binding_bind_property (
@@ -290,6 +357,41 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 
 	gtk_widget_show_all (GTK_WIDGET (m365_backend->priv->oauth2_settings_grid));
 
+	camel_m365_settings_lock (m365_settings);
+
+	gtk_expander_set_expanded (GTK_EXPANDER (expander),
+		(e_util_strcmp0 (camel_m365_settings_get_oauth2_endpoint_host (m365_settings), NULL) != 0 &&
+		 e_util_strcmp0 (camel_m365_settings_get_oauth2_endpoint_host (m365_settings), MICROSOFT365_ENDPOINT_HOST) != 0) ||
+		(e_util_strcmp0 (camel_m365_settings_get_oauth2_redirect_uri (m365_settings), NULL) != 0 &&
+		 e_util_strcmp0 (camel_m365_settings_get_oauth2_redirect_uri (m365_settings), MICROSOFT365_REDIRECT_URI) != 0));
+
+	camel_m365_settings_unlock (m365_settings);
+
+	e_binding_bind_property (
+		expander, "expanded",
+		advanced_help, "visible",
+		G_BINDING_SYNC_CREATE);
+
+	e_binding_bind_property (
+		expander, "expanded",
+		endpoint_host_label, "visible",
+		G_BINDING_SYNC_CREATE);
+
+	e_binding_bind_property (
+		expander, "expanded",
+		m365_backend->priv->oauth2_endpoint_host_entry, "visible",
+		G_BINDING_SYNC_CREATE);
+
+	e_binding_bind_property (
+		expander, "expanded",
+		redirect_uri_label, "visible",
+		G_BINDING_SYNC_CREATE);
+
+	e_binding_bind_property (
+		expander, "expanded",
+		m365_backend->priv->oauth2_redirect_uri_entry, "visible",
+		G_BINDING_SYNC_CREATE);
+
 	e_binding_bind_object_text_property (
 		settings, "user",
 		m365_backend->priv->user_entry, "text",
@@ -315,11 +417,17 @@ mail_config_m365_backend_insert_widgets (EMailConfigServiceBackend *backend,
 		G_BINDING_SYNC_CREATE);
 
 	e_binding_bind_object_text_property (
-		settings, "oauth2-redirect_uri",
+		settings, "oauth2-redirect-uri",
 		m365_backend->priv->oauth2_redirect_uri_entry, "text",
 		G_BINDING_BIDIRECTIONAL |
 		G_BINDING_SYNC_CREATE);
 
+	e_binding_bind_object_text_property (
+		settings, "oauth2-endpoint-host",
+		m365_backend->priv->oauth2_endpoint_host_entry, "text",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
+
 	extension_name = E_SOURCE_EXTENSION_COLLECTION;
 	source = e_mail_config_service_backend_get_collection (backend);
 	extension = e_source_get_extension (source, extension_name);
@@ -412,15 +520,19 @@ mail_config_m365_backend_check_complete (EMailConfigServiceBackend *backend)
 		CamelM365Settings *m365_settings = CAMEL_M365_SETTINGS (settings);
 		const gchar *client_id;
 
+		camel_m365_settings_lock (m365_settings);
+
 		if (camel_m365_settings_get_override_oauth2 (m365_settings)) {
 			client_id = camel_m365_settings_get_oauth2_client_id (m365_settings);
 		} else {
 			client_id = MICROSOFT365_CLIENT_ID;
 		}
 
-		correct = client_id && *client_id;
+		correct = e_util_strcmp0 (client_id, NULL) != 0;
 		complete = complete && correct;
 
+		camel_m365_settings_unlock (m365_settings);
+
 		e_util_set_entry_issue_hint (m365_backend->priv->oauth2_client_id_entry, correct ? NULL : _("Application ID cannot be empty"));
 	}
 
